function output = radial_functions_prime( type, n, x )

% radial_functions --> Evaluate spherical Bessel/Hankel functions of different kinds
%   
%   This program calculates the spherical versions of the Bessel and Hankel functions of first and second kinds for a set of values x corresponding to the 
% dependent variable. This is done by relating the spherical versions to their non-spherical counterparts. These four types of functions are of particular 
% interest to expressing solutions of the Wave Equation as an infinite series summation of harmonic terms (spherical or cylindrical).
%
%
%   type = string input that is used to choose which of the four cases of spherical functions to be evaluated; must be one of the following:
%             -> 'j'  = spherical Bessel function of the first kind;                                   
%             -> 'y'  = spherical Bessel function of the second kind;
%             -> 'h1' = spherical Hankel function of the first kind;
%             -> 'h2' = spherical Hankel function of the second kind;
%   
%   n = order of the spherical Bessel/Hankel function; does not have to be integer;
%
%   x = vector of values corresponding to the dependent variable;
%
%   output =  the variable in which the evaluated values of x are stored;
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Written by Mihai Orita (SPCG, ISVR, Southampton), in the period July 2017  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Last updated on 21/07/2017, performed by Mihai Orita                       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


%% INPUT VARIABLE VERIFICATION

if isempty(find(strcmp(type,{'j' 'y' 'h1' 'h2'}), 1) ~= 0) == 1            % Check if input variable type is introduced correctly;
    
    error('Input variable TYPE must be introduced as one of the strings: "j", "y", "h1", or "h2" !');
    
end

if isnumeric(n) == 0 || isnumeric(x) == 0                                  % Check if input variables n and x are numeric;
    
    error('Input variables n and x must be numeric!');
    
end

if isscalar(n) == 0                                                        % Check if input variable n is a scalar;
    
    error('Input variable n must be numeric!');
    
end


%% FUNCTION EVALUATION

if strcmp(type,'j') == 1                                                   % Choice is spherical Bessel function of the first kind;
    
    j_n = sqrt(pi./(2*x)).*besselj(n+1/2,x);                               % Calculate and output spherical version of the function by relating it to the   
    j_n1 = sqrt(pi./(2*x)).*besselj(n-1+1/2,x);
    output = j_n1-((n+1)./x).*j_n;                                                          % non-shperical version, for a vector of values x of the dependent variable;  
    

end

if strcmp(type,'y') == 1                                                   % Choice is spherical Bessel function of the second kind;
    
    y_n = sqrt(pi./(2*x)).*bessely(n+1/2,x);                               % Calculate and output spherical version of the function by relating it to the   
    y_n1 = sqrt(pi./(2*x)).*bessely(n-1+1/2,x);
    output = y_n1-((n+1)./x).*y_n;                                                          % non-spherical version, for a vector of values x of the dependent variable;

end

if strcmp(type,'h1') == 1                                                  % Choice is spherical Hankel function of the first kind;
    
    h1_n = sqrt(pi./(2*x)).*(besselj(n+1/2,x)+1i*bessely(n+1/2,x));        % Calculate and output spherical version of the function by relating it to the   
    h1_n1 = sqrt(pi./(2*x)).*(besselj(n-1+1/2,x)+1i*bessely(n-1+1/2,x));
    output = h1_n1-((n+1)./x).*h1_n;                                                         % spherical Bessel functions of first and second kind, for a vector of values x 
                                                                           % of the dependent variable;
end


if strcmp(type,'h2') == 1                                                  % Choice is spherical Hankel function of the second kind;                                     
    
    h2_n = sqrt(pi./(2*x)).*(besselj(n+1/2,x)-1i*bessely(n+1/2,x));        % Calculate and output spherical version of the function by relating it to the       
    h2_n1 = sqrt(pi./(2*x)).*(besselj(n-1+1/2,x)-1i*bessely(n-1+1/2,x));
    output = h2_n1-((n+1)./x).*h2_n;                                                         % spherical Bessel functions of first and second kind, for a vector of values x                                                     
                                                                           % of the dependent variable;                                                            
end


end
