function Y_n = spherical_harmonics( n, theta, phi )

% decoding_matrix --> Calculates the decoding matrix required in Ambisonics audio reproduction, for a given loudspeaker arrangement
% 
%   This program is based on Ambisonics audio reproduction where the sound field is assumed to be composed only of arriving plane waves  
%   (acoustic sources and reproduction devices are far away from the listening position + interior problem). The decoding matrix required 
%   in the reproduction algorithm is calculated from the positions of a loudspeaker arrangement, which are described as azimuth and eleveation 
%   angles in an input text  file. The real valued spherical harmonics are used for this program. Representations of how the text file must 
%   be constructed and, respectively, of the decoding matrix are shown below.
%   
%   n = the order of Ambisonics decoding;
%     = is the highest order of spherical harmonics used in the linear system of equations from which the decoding matrix originates; 
%
%   L = number of reproduction devices;
%
%   spatial_info = matrix that stores the information regarding the positions of the reproduction devices;
%                = it is an 2xL matrix, where the first column represents theta (co-elevation angle), the second column represents phi (azimuth 
%                  angle), and a row represents a reproduction device;  
%                = the information is read from the text file titled 'positions_reproduction_devices.txt' and must be given in degrees;
%
%     |----------------------|----------------------|
%     | Theta (co-elevation) |     Phi (azimuth)    |            
%     |----------------------|----------------------|
%     |                      |                      |  
%     |----------------------|----------------------|
%     |                      |                      |
%     |----------------------|----------------------|
%     |                      |                      |
%     |----------------------|----------------------|
%     |                      |                      |          
%     |----------------------|----------------------|
%     |           :          |           :          |           
%     |           :          |           :          |          
%     |           :          |           :          |    

%
%   D = variable in which the decoding matrix is stored at the end of the algorithm;
%     = the decoding matrix is of dimension Lx(n+1)^2, where the rows represent all the spherical harmonics dictated by the maximum considered 
%       order n, and the columns represent the reproduction devices;
%
%     |------------------|---------|------------------------------|---------------------------------------------------|----------  
%     |        \ Order   | Order 0 |            Order 1           |                       Order 2                     |
%     |------------------|---------|----------|---------|---------|----------|----------|---------|---------|---------|----------  
%     |        \ Degree  |  m = 0  |   m = 1  |  m = 0  |  m = 1  |   m = 2  |   m = 1  |  m = 0  |  m = 1  |  m = 2  | .. .. ..
%     |------------------|---------|----------|---------|---------|----------|----------|---------|---------|---------|----------
%     | Source \ Version |    -    |   sine   |    -    | cosine  |   sine   |   sine   |    -    | cosine  | cosine  |
%     |------------------|---------|----------|---------|---------|----------|----------|---------|---------|---------|----------     
%     |      Source 1    |         |          |         |         |          |          |         |         |         |
%     |------------------|---------|----------|---------|---------|----------|----------|---------|---------|---------|----------
%     |      Source 2    |         |          |         |         |          |          |         |         |         |
%     |------------------|---------|----------|---------|---------|----------|----------|---------|---------|---------|----------  
%     |      Source 3    |         |          |         |         |          |          |         |         |         |
%     |------------------|---------|----------|---------|---------|----------|----------|---------|---------|---------|----------  
%     |         :        |         |          |         |         |          |          |         |         |         |  
%     |         :        |         |          |         |         |          |          |         |         |         |
%     |         :        |         |          |         |         |          |          |         |         |         |
%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Written by Mihai Orita  (SPCG, ISVR, Southampton), in the period July 2017  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Last updated on 21/07/2017, performed by Mihai Orita                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 


%spatial_info = load('positions_reproduction_devices.txt');                       % read the positions of the audio reproduction devices from text file;
%
%L = numel(spatial_info)/2;                                                       % work out the number of audio reproduction devices from text file;

%Y = zeros ((n+1)^2,numel(theta));                                                                 % initialize decoding matrix;

%harmonics_counter = 0;                                                           % variable for counting the number of harmonics that have already been 
                                                                                 % calculated and stored in D;

%for order = 0:n                                                                  % calculate harmonics for each order from 0 to n;
    
    
    N_pos_degrees = legendre(n,cos(theta),'norm');                     % associated Legendre functions for all positive degrees (including  
                                                                                 % zero) up to n; the result is a (n+1)xL matrix, where a row represents 
%     var1 = (-order:1:-1)';                                                                             % a degree and a column represents a reproduction device;
%     var2 = (-1).^(-var1).*factorial(order-(-var1))./factorial(order+(-var1));
%     P_neg_degrees = bsxfun(@times, var2, P_pos_degrees(end:-1:2,:));
%     P = [P_neg_degrees; P_pos_degrees];
%     
%     var3 = (0:order)';
%     var4 = sqrt(((2*order+1)/(4*pi))*factorial(order-var3)./factorial(order+var3));
%     var5 = bsxfun(@times, ones(order+1,numel(phi)), phi);
%     
%     var6 = bsxfun(@times, var3, var5); 
%     var7 = bsxfun(@times, var4, exp(1i*var6));
%     var8 = bsxfun(@times, var7, P_pos_degrees);

    
    var1 = (0:n)';
    var2 = bsxfun(@times, ones(n+1,numel(phi)), phi);
    
    var3 = bsxfun(@times, var1, var2); 
    var4 = bsxfun(@times, 1./((-1).^var1), N_pos_degrees/(sqrt(2*pi)));
    var5 = bsxfun(@times, var4, exp(1i*var3));

    var6 = (-n:1:-1)';
    var7 = var5(end:-1:2,:);
    var8 = bsxfun(@times, (-1).^abs(var6), conj(var7));
    
    Y_n = [var8;var5];

                                                                                 
%     if order ~= 0                                                                % calculate 0-th order harmonics separately;
% 
%         P = [P_pos_degrees(end:-1:2,:); P_pos_degrees];                          % use matrix of associated Legendre functions for both cosine and sine 
%                                                                                  % versions of the spherical harmonics; in the end, the order of the  
%                                                                                  % degrees (rows in matrix) are {...3,2,1,0,1,2,3...};                                                                                         
%                                                                                                                 
%         degrees = [order:-1:0 1:order];  
%         var1 = (-1).^degrees*sqrt(2).*sqrt((2*order+1)*factorial(order-degrees)./(factorial(order+degrees)*(4*pi)));  
%         
%         C = bsxfun(@times, var1', P);                                            % calculate and apply scaling factor to associated Legendre functions             
%                                                                                                                                                                    
%         C(order+1,:) = C(order+1,:)/sqrt(2);                                     % account for a different scaling factor when degree is 0                                  
%                                                                                                                                                                    
%         cos_or_sin = [sind(degrees(1:order)'*spatial_info(:,2)'); ...            % the order of the versions in the matrix is                                                                                          
%                       cosd(degrees(order+1:2*order+1)'*spatial_info(:,2)')];     % {...sine,sine,zero,cosine,cosine...}  
%                                                                                                                                                                                                                                           
%         C = bsxfun(@times, cos_or_sin, C);                                       % multiply by cosine or sine                                                      
%         
%     else
%         
%         P = P_pos_degrees;                                                       % for an order of zero, only one row is calculated to complete the matrix  
%         C = sqrt(1/(4*pi)) .* P;                                                 % D and the spherical harmonics doesn`t have a cosine nor a sine term;
%         
%     end

    %[current_count, ~] = size(Y_n);
    
    
                                                                                 % fill matrix D progresively; for each loop, the spherical harmonics of a  
                                                                                 % order are calculated and added to the previous set in D;

    %Y(harmonics_counter+1:(harmonics_counter+current_count),:) = Y_n;
    %harmonics_counter = harmonics_counter + current_count;
    
%     if harmonics_counter ~= 1
%         
%         Y(harmonics_counter+1:(harmonics_counter+current_count),:) = Y_n;
%         harmonics_counter = harmonics_counter + current_count;
%          
%     else
%         
%         Y(harmonics_counter:current_count,:) = Y_n;
%         
%     end
    %harmonics_counter = harmonics_counter + current_count;               % each loop produces (order+1)^2 - order^2 new spherical harmonics to be 
                                                                                 % added in D;
%end


%D = pinv(D);                                                                     % calculate pseudo-inverse to obtain decoding matrix in D;   

end

